/**
 * FAQhere is a simple and small tool to show frequently asked questions
 * within an existing web service or web site. The FAQs are show (and ranked)
 * in the context on the web site.
 * 
 * Currently MySQL
 * 
 * @author Dominik Leiner, Munich, info@msd-media.de
 * 
 * This software is licenced under GPL and
 * can be found at http://faqhere.sourceforge.net
 * 
 * 
 * How use
 * -------
 * - Modify config.php to change admin password, database settings, and (optional) path to PHP
 * - Run faq.install.php to set up database tables (you must unlock this file first by changing a significantly marked line within)
 * - See dummy.html for implementation on web site (faq.php my be under the same domain like the html not to create a security violation using ajax)
 * - The context is defined in the JavaScript new miniFAQ() - see dummy.html
 * 
 * This module has initially been developed for the online survey software
 * oFb - der onlineFragebogen (http://ofb.msd-media.de)
 */

/**
 * Reaction to a FAQ search,
 * should be performed when clicking on search-submit.
 * @param String searchFor
 */
function miniFAQ(context, url) {
	this.serverURL;
	if ((typeof url != "string") || (url.length < 1)) {
		this.serverURL = "";
	} else {
		var test = url.substr(url.length-7);
		if (test != "faq.php") {
			alert("Server URL must end on faq.php");
			return false;
		}
		this.serverURL = url.substring(0, url.length-7);
	}
	var currentContext = context;	// String  Current FAQ context
	var htmlAnchor = null;			// HTMLElement  FAQ anchor image
	var htmlOutput = null;			// HTMLElement  FAQ output box
	var defaultListing = null;		// Cache for default FAQs for the page
	var answerCache = new Object(); // Cache for known answers
	var ratingCache = new Object(); // Cache for ratings
	var localThis = this;
	var mouseTimeout;
	
	this.setting = new Object();
	this.setting["show credits"] = "yes";
	
	this.language = new Object();
	this.language["search for answer"] = "Search answer";
	this.language["no questions found"] = "No FAQs found matching your request";
	this.language["rating"] = "Rate this answer";
	this.language["helpful"] = "helpful";
	this.language["not helpful"] = "not helpful";
	
	/**
	 * Search for FAQs matching given text.
	 * Will return default FAQs for the page if no text is given.
	 * @param String text  Text to perform full text search on
	 */
	this.search = function(text, storeListingAsDefault) {
		var data = new Object();
		data["do"] = "search";
		data["c"] = currentContext;
		if (text !== false) {
			data["s"] = text;
		}
		var info = new Object();
		info["store as default"] = storeListingAsDefault
		this.setStatus(true);
		var request = new miniAjax();
		request.sendRequestGET(this.serverURL+"faq.php", data, this.searchCallback, info);
	}
	
	/**
	 * Event handler
	 */
	this.searchDefault = function(evt) {
		if (mouseTimeout) {
			window.clearTimeout(mouseTimeout);
			mouseTimeout = null;
		}
		if (defaultListing !== null) {
			// Use cache and do not ask again
			localThis.showListing(defaultListing);
		} else {
			localThis.search(false, true);
		}
	}
	
	/**
	 * Stop a timer to show searchDefault
	 */
	function searchDefaultTimerStop(evt) {
		if (mouseTimeout) {
			window.clearTimeout(mouseTimeout);
			mouseTimeout = null;
		}
	}
	 
	/**
	 * Start a timer to show searchDefault
	 */
	function searchDefaultTimerStart(evt) {
		if (mouseTimeout) {
			// Already running
			return;
		}
		mouseTimeout = window.setTimeout(localThis.searchDefault, 200);
	}

	this.searchCallback = function(request, info) {
		var storeListingAsDefault = info["store as default"];
		localThis.setStatus(false);
		// alert(request.responseText);
		if (!request.responseXML) {
			alert("No XML response received\n\n" + request + "\n\n" + request.responseText);
			return;
		}
		var answer = request.responseXML;
		var faqAnswers = answer.getElementsByTagName("faq-answer");
		if (faqAnswers.length < 1) {
			alert("Received invalid XML answer from the server");
			return;
		}
		var faqAnswer = faqAnswers[0];
		
		// Read answer
		var questionListings = faqAnswer.getElementsByTagName("listing");
		if (questionListings.length < 1) {
			alert("Server answer contained no listing!");
			return;
		}
		var questionListing = questionListings[0];
		if (storeListingAsDefault) {
			defaultListing = questionListing;
			var languageNodes = faqAnswer.getElementsByTagName("lang");
			localThis.storeLanguage(languageNodes);
			var settingNodes = faqAnswer.getElementsByTagName("set");
			localThis.storeSettings(settingNodes);
		}
		
		// Store answers ans show listing
		localThis.storeAnswers(questionListing);
		localThis.showListing(questionListing);
		return questionListing;
	}
	
	this.setStatus = function(isWaiting) {
		var button = htmlAnchor;
		if (isWaiting) {
			button.src = this.serverURL+"faq.waiting.gif";
		} else {
			button.src = this.serverURL+"faq.gif";
		}
	}
	
	/**
	 * Store any answers contained in the answer in answerCache.
	 */
	this.storeAnswers = function(listing) {
		for (var i=0; i<listing.childNodes.length; i++) {
			if (listing.childNodes[i].nodeType != 1) continue;
		
			var id = listing.childNodes[i].getAttribute("id");
			var nodeName = listing.childNodes[i].nodeName;
			var nodeValue = listing.childNodes[i].firstChild.nodeValue;
			
			if (nodeName == "answer") {
				answerCache[id] = nodeValue;
			}
		}
	}
	
	/**
	 * Store language elements (phrases) sent in an XML answer.
	 */
	this.storeLanguage = function(nodes) {
		for (var i=0; i<nodes.length; i++) {
			var id = nodes[i].getAttribute("id");
			var content = nodes[i].firstChild.nodeValue;
			this.language[id] = content;
		}
	}
	
	/**
	 * Store settings sent in an XML answer.
	 */
	this.storeSettings = function(nodes) {
		for (var i=0; i<nodes.length; i++) {
			var id = nodes[i].getAttribute("id");
			var content = nodes[i].firstChild.nodeValue;
			this.setting[id] = content;
		}
	}
	
	/**
	 * Return a specific setting.
	 * @param String id  Setting ID
	 * @return String  Setting value
	 */
	this.getSetting = function(id) {
		return this.setting[id];
	}
	
	/**
	 * Show a given listing of FAQs.
	 */
	this.showListing = function(listing) {
		this.removeOutput();
		
		var questions = listing.getElementsByTagName("question");
		if (questions.length < 1) {
			alert(this.language["no questions found"]);
			return;
		}
		for (var i=0; i<listing.childNodes.length; i++) {
			if (listing.childNodes[i].nodeType != 1) continue;
		
			var id = listing.childNodes[i].getAttribute("id");
			var nodeName = listing.childNodes[i].nodeName;
			var nodeValue = listing.childNodes[i].firstChild.nodeValue;
			
			if (nodeName == "question") {
				var rating = listing.childNodes[i].getAttribute("rating");
				htmlOutput.addQuestion(nodeValue, id, rating);
			} else if (nodeName == "kategory") {
				htmlOutput.addKategory(nodeValue, id);
			} else {
				// Ignore
			}  
		}
		
		// Add copyright if set so
		if (this.getSetting("credits") == "yes") {
			var cp = document.createElement("p");
			cp.className = "faqCredits";
			var ca = document.createElement("a");
			ca.setAttribute("href", "http://faqhere.sourceforge.net");
			var ct1 = document.createTextNode("powered by ");
			var ct2 = document.createTextNode("FAQhere");
			ca.appendChild(ct2);
			cp.appendChild(ct1);
			cp.appendChild(ca);
			htmlOutput.addHTMLNode(cp);
		}
	}
	
	this.removeOutput = function() {
		// Remove previous output
		if (htmlOutput !== null) {
			htmlOutput.remove();
			htmlOutput = null;
		}
		htmlOutput = new miniFAQBox(this);
	}
	
	this.button = function() {
		currentContext = context;
		document.writeln('<img src="'+this.serverURL+'faq.gif" id="miniFAQ_anchor">');
		htmlAnchor = document.getElementById("miniFAQ_anchor");
		htmlAnchor.onmouseover = searchDefaultTimerStart;
		htmlAnchor.onmouseout = searchDefaultTimerStop;
		htmlAnchor.onclick = this.searchDefault;
	}
	
	/**
	 * Retrieve an answer from the database (or from local cache)
	 * and call back callback function.
	 * @param Function callback(id, answer)
	 */
	this.retrieveAnswer = function(id, callback) {
		// Check cache
		if (answerCache[id]) {
			var answer = answerCache[id];
			callback(id, answer);
			return;
		}
		// Ask for answer
		var data = new Object();
		data["do"] = "get";
		data["id"] = id;
		data["c"] = currentContext;
		var request = new miniAjax();
		var info = new Object();
		info["id"] = id;
		info["callback"] = callback;
		request.sendRequestGET(this.serverURL+"faq.php", data, this.retrieveAnswerCallback, info);
	}
	
	/**
	 * Handler for the XML answer from this.retrieveAnswer.
	 * @param HTTPRequest request
	 * @param Object info
	 * 			Integer  id                    Question ID
	 * 			Function callback(id, answer)  Function to call when answer was found
	 */
	this.retrieveAnswerCallback = function(request, info) {
		var xmlAnswer = request.responseXML;
		var faqAnswers = xmlAnswer.getElementsByTagName("faq-answer");
		if (faqAnswers.length < 1) {
			alert("Received invalid XML answer from the server:\n"+request.responseText);
			return;
		} else {
			var faqAnswer = faqAnswers[0];
			localThis.storeAnswers(faqAnswer);
		}
		
		var callback = info["callback"];
		var id = info["id"];
		
		// Check if we have an answer
		if (answerCache[id]) {
			var answer = answerCache[id];
			callback(id, answer);
		} else {
			alert("Did not receive an answer for FAQ "+id);
		}
	}
	
	/**
	 * Check if an answer has already been rated.
	 */
	this.checkRating = function(id) {
		if ((ratingCache[id]) && (ratingCache[id] != 0)) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Remember that a question has already been rated.
	 * (Till the next page reload).
	 */
	this.setRating = function(id, rating) {
		ratingCache[id] = rating;
		var data = new Object();
		data["do"] = "rate";
		data["c"] = currentContext;
		data["id"] = id;
		data["rating"] = rating;
		var request = new miniAjax();
		var callback = function() {}
		request.sendRequestGET(this.serverURL+"faq.php", data, callback);
	}
	
	this.button();
}


function miniAjax() {
	
	this.request = null;
	
	this.initRequestObject = function() {
		this.request = false;
		// Try latest implementation
		if (typeof XMLHttpRequest != 'undefined') {
		    this.request = new XMLHttpRequest();
		}
		// Try IE if necessary
		if (!this.request) {
		    try {
		    	this.request = new ActiveXObject("Msxml2.XMLHTTP");
			} catch(e) {
		    	try {
					this.request = new ActiveXObject("Microsoft.XMLHTTP");
				} catch(e) {
				    this.request = false;
				}
		    }
		}
	}
	
	/**
	 * @param String url   URL to call
	 * @param Object data  Assotiative array of data to send
	 * @param func callback         Function to call when result has arrived arguments: (request-object, additionalData)
	 * @param mixed additionalData  Any additional data that should be send to the callback function
	 */
	this.sendRequestGET = function(url, data, callback, additionalData) {
		var performCallback;
		if (callback === null) {
			// No callback required
			performCallback = false;
		} else if (typeof(callback) != "function") {
			alert("Expected function as callback argument, but got type "+typeof(callback)+"\n\n"+callback);
			return;
		} else {
			performCallback = true;
		}
		// Build URL+GET
		var urlWithQuery = url+"?"+this.encodeDataToGET(data);
		// Send
		var request = this.request;
		if (performCallback) {
			request.onreadystatechange = function() {
				if (request.readyState != 4) {
					return false;
				}
				callback(request, additionalData);
			}
		}
		request.open("GET", urlWithQuery, true);
		request.send(null);
	}
	
	/**
	 * Send synchronous GET request 
	 */
	this.sendRequestGETSync = function(url, job) {
		// Build URL+GET
		var urlWithQuery = url+"?"+job.encodeGET();
		// Send
		this.request.open("GET", urlWithQuery, false);
		this.request.send(null);
		// Return as request has been finished
	}
	
	/**
	 * Encode a given object (associative array) to GET string.
	 */
	this.encodeDataToGET = function(data) {
		var get = "";
		for (var n in data) {
			var transName = n;
			// Data that is false will be encoded as empty string
			var transData = data[n];
			if (transData == false) {
				transData = "";
			}
			// Browsers recognize ��� as multibytes using encodeURIComponent() -> need escape()
			get +=
				"&"+
				escape(transName)+
				"="+
				escape(transData);
		}
		return get;
	}
	
	// Constructor: Automatically create XMLHttpRequest Object
	this.initRequestObject();
}

/**
 * Shows a box with FAQs and manages their activation.
 */
function miniFAQBox(miniFAQ) {
	
	var bodyNodes = document.getElementsByTagName("BODY");
	var bodyNode = bodyNodes[0];
	var activeQuestion = null;
	var activeAnswer = null;
	var localThis = this;
	var faqParent = miniFAQ;
	var searchInput = null;
	var searchTimeout = null;
	
	/**
	 * Close box.
	 * Event handler.
	 */
	this.close = function(event) {
		localThis.remove();
	}
	
	/**
	 * Search for given string.
	 * Event handler.
	 */
	this.search = function(event) {
		var s = searchInput.value;
		faqParent.search(s, false);
		return false;
	}
	
	/**
	 * Check if return key was pressed to start searching.
	 */
	this.searchKeydown = function(event) {
		if (!event) {
			event = window.event;
		}
		if (event.keyCode == 13) {
			localThis.search();
			return false;
		} else {
			return true;
		}
	}
	 
	/**
	 * Start a timer to start searching after not typing for 1 second.
	 */
	this.searchAutomatically = function(event) {
		// Do not search automatically because it may show an error message if nothing is found
		return;
		// Stop a running timer
		if (searchTimeout != null) {
			window.clearTimeout(searchTimeout);
			searchTimeout = null;
		}
		searchTimeout = window.setTimeout(localThis.search, 1000);
	}
	
	// Create box and append to HTML tree
	var outputNode = document.createElement("div");
	outputNode.className = "faqBox";
	
	// Modify position and width if there is not enough room
	var availableWidth = 1024;
	if (window.innerWidth) {
		availableWidth = window.innerWidth;
	} else if (document.documentElement) {
		availableWidth = document.documentElement.clientWidth;
	} else if (document.body) {
		availableWidth = document.body.clientWidth;
	}
	if (availableWidth < 560) {
		var left = (availableWidth - 320) / 2;
		if (left < 0) {
			left = 0;
			var width = availableWidth - 20;
			outputNode.style.width = width+"px";
		}
		outputNode.style.left = left+"px";
	}
	
	
	// Show as pop up
	outputNode.style.zIndex = "1024";
	bodyNode.appendChild(outputNode);
	
	// Show search area
	var searchForm = document.createElement("form");
	searchForm.className = "faqSearch";
	searchForm.onsubmit = this.search;
	
	var searchText = document.createTextNode(faqParent.language["search for answer"]+": ");
	searchInput = document.createElement("input");
	searchInput.className = "faqSearch";
	searchInput.onkeyup = this.searchAutomatically;
	searchInput.onkeydown = this.searchKeydown;
	
	var closeButton = document.createElement("button");
	closeButton.setAttribute("id", "faqBTClose");
	closeButton.setAttribute("type","button");
	closeButton.onclick = searchForm.onsubmit;
	var closeButtonImg = document.createElement("img");
	closeButtonImg.src = faqParent.serverURL+"faq.close.gif";
	closeButton.appendChild(closeButtonImg);
	closeButton.onclick = this.close;
	var searchButton = document.createElement("button");
	searchButton.setAttribute("id", "faqBTSearch");
	searchButton.setAttribute("type","submit");
	searchButton.onclick = searchForm.onsubmit;
	var searchButtonText = document.createTextNode("OK");
	searchButton.appendChild(searchButtonText);
	
	formSearch = document.createElement("div");
	formSearch.setAttribute("id", "faqSearch");
	
	// Assemble
	formSearch.appendChild(closeButton);
	formSearch.appendChild(searchText);
	formSearch.appendChild(searchInput);
	formSearch.appendChild(searchButton);
	
	outputNode.appendChild(formSearch);
	
	// collect questions
	var questions = new Array();
	
	this.addQuestion = function(text, id, rating) {
		var qp = document.createElement("p");
		qp.className = "faqQuestion";
		var questionText = text;
		if (rating) {
			questionText += " ("+rating+")";
		}
		var qt = document.createTextNode(questionText);
		qp.appendChild(qt);
		outputNode.appendChild(qp);
		
		qp.questionId = id;
		qp.onclick = this.careForAnswer;
	
		var newQuestion = new Object();
		newQuestion["text"] = text;
		newQuestion["id"] = id;
		newQuestion["rating"] = rating;
		newQuestion["node"] = qp;
		questions.push(newQuestion);
	}
	
	/**
	 * Add any HTML node.
	 */
	this.addHTMLNode = function(node) {
		outputNode.appendChild(node);
	}
	
	
	this.addKategory = function(text, id) {
		var kp = document.createElement("p");
		kp.className = "faqKategory";
		var kt = document.createTextNode(text);
		kp.appendChild(kt);
		outputNode.appendChild(kp);
	}
	
	/**
	 * Show the answer to a frequently asked question.
	 * @param int id  Question id to open
	 */
	this.showAnswer = function(id, answer) {
		// Remove last answer shown
		if (activeAnswer !== null) {
			outputNode.removeChild(activeAnswer);
			activeQuestion["node"].className = "faqQuestion";
			activeQuestion = null;
			activeAnswer = null;
		}
		// Find question
		var question = null;
		for (var i=0; i<questions.length; i++) {
			if (questions[i]["id"] == id) {
				question = questions[i];
				break;
			}
		}
		if (question === null) {
			return;
		}
		// Highlight question
		question["node"].className = "faqQuestion faqActive";
		// Show new answer
		var ad = document.createElement("div");
		var ap = document.createElement("div");
		var html = answer;
		ap.className = "faqAnswer faqActive";
		ap.innerHTML = html;
		ad.appendChild(ap);
		// Add before next title or at the end of the outputNode
		var sibling = question["node"].nextSibling;
		if (sibling) {
			outputNode.insertBefore(ad, sibling);
		} else {
			outputNode.appendChild(ad);
		}
		
		// Rating option
		var alreadyRated = faqParent.checkRating(id);
		var rp = null;
		if (!alreadyRated) {
			rp = document.createElement("p");
			rp.className = "faqRating";
			var r2t = document.createTextNode("["+faqParent.language["helpful"]+"]");
			var r3t = document.createTextNode("["+faqParent.language["not helpful"]+"]");
			var rst = document.createTextNode(" ");
			var r2a = document.createElement("a");
			var r3a = document.createElement("a");
			r2a.onclick = function(event) { localThis.rate(id, +1);	}
			r3a.onclick = function(event) { localThis.rate(id, -1);	}
			r2a.appendChild(r2t);
			r3a.appendChild(r3t);
			rp.appendChild(r2a);
			rp.appendChild(rst);
			rp.appendChild(r3a);
			ad.appendChild(rp);
		}
		
		// Set new active question
		activeQuestion = question;
		activeAnswer = ad;
		activeRating = rp;
	}
	
	this.rate = function(id, rating) {
		if (activeRating == null) return;
		if ((rating == 1) || (rating == -1)) {
			// Remove rating link
			var arParent = activeRating.parentNode;
			arParent.removeChild(activeRating);
			// Remember rating & send to server
			faqParent.setRating(id, rating);
		}
	}
	
	/**
	 * Event handler.
	 * Tells faqParent to retrieve the answer from database and
	 * call back localThis.showAnswer(id, answer).
	 */
	this.careForAnswer = function(event) {
		this.className = "faqQuestion faqActive";
		faqParent.retrieveAnswer(this.questionId, localThis.showAnswer);
	}
	
	/**
	 * Remove the box from the HTML tree.
	 */
	this.remove = function() {
		if (outputNode !== null) {
			bodyNode.removeChild(outputNode);
			outputNode = null;
		}	
	}
	
}